@@ -2,7 +2,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/ioctl.h>
+#include <sys/signal.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <termios.h>
@@ -30,8 +30,9 @@ static char * helplineStack[20];
 static char ** currentHelpline = NULL;
 
 static int cursorRow, cursorCol;
+static int needResize;
 
-static char * defaultHelpLine = 
+static const char * defaultHelpLine = 
 "  <Tab>/<Alt-Tab> between elements   |  <Space> selects   |  <F12> next screen"
 ;
 
@@ -105,7 +106,7 @@ static struct keymap keymap[] = {
 };
 static char keyPrefix = '\033';
 
-static char * version = "Newt windowing library version " VERSION
+static const char * version = "Newt windowing library version " VERSION
 			" - (C) 1996 Red Hat Software. "
 		        "Redistributable under the term of the Library "
 		        "GNU Public License. "
@@ -117,6 +118,14 @@ void newtSetSuspendCallback(newtSuspendCallback cb) {
     suspendCallback = cb;
 }
 
+static void handleSigwinch(int signum) {
+    needResize = 1;
+}
+
+static int getkeyInterruptHook(void) {
+    return -1;
+}
+
 void newtFlushInput(void) {
     while (SLang_input_pending(0)) {
 	SLang_getkey();
@@ -146,19 +155,31 @@ void newtCls(void) {
     newtRefresh();
 }
 
+void newtResizeScreen(int redraw) {
+    newtPushHelpLine("");
+
+    SLtt_get_screen_size();
+    SLang_init_tty(0, 0, 0);
+
+    SLsmg_touch_lines (0, SLtt_Screen_Rows - 1);
+
+    /* I don't know why I need this */
+    SLsmg_refresh();
+
+    newtPopHelpLine();
+
+    if (redraw)
+	SLsmg_refresh();
+}
+
 int newtInit(void) {
-    struct winsize ws;
     char * MonoValue, * MonoEnv = "NEWT_MONO";
 
     /* use the version variable just to be sure it gets included */
     strlen(version);
 
     SLtt_get_terminfo();
-
-    if (!ioctl(1, TIOCGWINSZ, &ws)) {
-	SLtt_Screen_Rows = ws.ws_row;
-	SLtt_Screen_Cols = ws.ws_col;
-    }
+    SLtt_get_screen_size();
 
     MonoValue = getenv(MonoEnv);
     if ( MonoValue == NULL ) {
@@ -175,6 +196,13 @@ int newtInit(void) {
 
     /*SLtt_set_cursor_visibility(0);*/
 
+    /*memset(&sa, 0, sizeof(sa));
+    sa.sa_handler = handleSigwinch;
+    sigaction(SIGWINCH, &sa, NULL);*/
+
+    SLsignal_intr(SIGWINCH, handleSigwinch);
+    SLang_getkey_intr_hook = getkeyInterruptHook;
+
     return 0;
 }
 
@@ -238,6 +266,14 @@ int newtGetKey(void) {
 
     do {
 	key = SLang_getkey();
+	if (key == 0xFFFF) {
+	    if (needResize)
+		return NEWT_KEY_RESIZE;
+
+	    /* ignore other signals */
+	    continue;
+	}
+
 	if (key == NEWT_KEY_SUSPEND && suspendCallback)
 	    suspendCallback();
     } while (key == NEWT_KEY_SUSPEND);
@@ -310,7 +346,7 @@ void newtClearKeyBuffer(void) {
 }
 
 int newtOpenWindow(int left, int top, int width, int height, 
-			  char * title) {
+			  const char * title) {
     int j, row, col;
     int n;
     int i;
@@ -353,7 +389,7 @@ int newtOpenWindow(int left, int top, int width, int height,
 	SLsmg_set_char_set(0);
 	SLsmg_write_char(' ');
 	SLsmg_set_color(NEWT_COLORSET_TITLE);
-	SLsmg_write_string(currentWindow->title);
+	SLsmg_write_string((char *)currentWindow->title);
 	SLsmg_set_color(NEWT_COLORSET_BORDER);
 	SLsmg_write_char(' ');
 	SLsmg_set_char_set(1);
@@ -376,6 +412,21 @@ int newtOpenWindow(int left, int top, int width, int height,
     return 0;
 }
 
+int newtCenteredWindow(int width, int height, const char * title) {
+    int top, left;
+
+    top = (SLtt_Screen_Rows - height) / 2;
+
+    /* I don't know why, but this seems to look better */ 
+    if ((SLtt_Screen_Rows % 2) && (top % 2)) top--;
+
+    left = (SLtt_Screen_Cols - width) / 2;
+
+    newtOpenWindow(left, top, width, height, title);
+
+    return 0;
+}
+
 void newtPopWindow(void) {
     int j, row, col;
     int n = 0;    
@@ -501,7 +552,7 @@ void newtRedrawHelpLine(void) {
     SLsmg_write_string(buf);
 }
 
-void newtPushHelpLine(char * text) {
+void newtPushHelpLine(const char * text) {
     if (!text)
 	text = defaultHelpLine;
     
@@ -527,7 +578,7 @@ void newtPopHelpLine(void) {
     newtRedrawHelpLine();
 }
 
-void newtDrawRootText(int row, int col, char * text) {
+void newtDrawRootText(int row, int col, const char * text) {
     SLsmg_set_color(NEWT_COLORSET_ROOTTEXT);
 
     if (col < 0) {
@@ -539,7 +590,7 @@ void newtDrawRootText(int row, int col, char * text) {
     }
    
     SLsmg_gotorc(row, col);
-    SLsmg_write_string(text);
+    SLsmg_write_string((char *)text);
 }
 
 int newtSetFlags(int oldFlags, int newFlags, enum newtFlagsSense sense) {
@@ -560,5 +611,10 @@ int newtSetFlags(int oldFlags, int newFlags, enum newtFlagsSense sense) {
 
 void newtBell(void)
 {
-	SLtt_beep();
+    SLtt_beep();
+}
+
+void newtGetScreenSize(int * cols, int * rows) {
+    if (rows) *rows = SLtt_Screen_Rows;
+    if (cols) *cols = SLtt_Screen_Cols;
 }
