@@ -5,6 +5,8 @@
 extern "C" {
 #endif
 
+#include <stdarg.h>
+
 #define NEWT_COLORSET_ROOT 		2
 #define NEWT_COLORSET_BORDER 		3
 #define NEWT_COLORSET_WINDOW		4
@@ -88,40 +90,43 @@ typedef void (*newtSuspendCallback)(void);
 int newtInit(void);
 int newtFinished(void);
 void newtCls(void);
+void newtResizeScreen(int redraw);
 void newtWaitForKey(void);
 void newtClearKeyBuffer(void);
 void newtDelay(int usecs);
 /* top, left are *not* counting the border */
 int newtOpenWindow(int left, int top, int width, int height, 
-			  char * title);
+			  const char * title);
+int newtCenteredWindow(int width, int height, const char * title);
 void newtPopWindow(void);
 void newtSetColors(struct newtColors colors);
 void newtRefresh(void);
 void newtSuspend(void);
 void newtSetSuspendCallback(newtSuspendCallback cb);
 void newtResume(void);
-void newtPushHelpLine(char * text);
+void newtPushHelpLine(const char * text);
 void newtRedrawHelpLine(void);
 void newtPopHelpLine(void);
-void newtDrawRootText(int row, int col, char * text);
+void newtDrawRootText(int row, int col, const char * text);
 void newtBell(void);
 
 /* Components */
 
-newtComponent newtCompactButton(int left, int top, char * text);
-newtComponent newtButton(int left, int top, char * text);
-newtComponent newtCheckbox(int left, int top, char * text, char defValue,
-			   char * seq, char * result);
-newtComponent newtRadiobutton(int left, int top, char * text, int isDefault,
+newtComponent newtCompactButton(int left, int top, const char * text);
+newtComponent newtButton(int left, int top, const char * text);
+newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
+			   const char * seq, char * result);
+newtComponent newtRadiobutton(int left, int top, const char * text, int isDefault,
 			      newtComponent prevButton);
 newtComponent newtRadioGetCurrent(newtComponent setMember);
-newtComponent newtListitem(int left, int top, char * text, int isDefault,
-			      newtComponent prevItem, void * data, int flags);
-void newtListitemSet(newtComponent co, char * text);
+newtComponent newtListitem(int left, int top, const char * text, int isDefault,
+			      newtComponent prevItem, const void * data, int flags);
+void newtListitemSet(newtComponent co, const char * text);
 void * newtListitemGetData(newtComponent co);
+void newtGetScreenSize(int * cols, int * rows);
 
-newtComponent newtLabel(int left, int top, char * text);
-void newtLabelSetText(newtComponent co, char * text);
+newtComponent newtLabel(int left, int top, const char * text);
+void newtLabelSetText(newtComponent co, const char * text);
 newtComponent newtVerticalScrollbar(int left, int top, int height,
 				    int normalColorset, int thumbColorset);
 void newtScrollbarSet(newtComponent co, int where, int total);
@@ -129,14 +134,16 @@ void newtScrollbarSet(newtComponent co, int where, int total);
 newtComponent newtListbox(int left, int top, int height, int flags);
 void * newtListboxGetCurrent(newtComponent co);
 void newtListboxSetCurrent(newtComponent co, int num);
-void newtListboxSetText(newtComponent co, int num, char * text);
-void newtListboxSetEntry(newtComponent co, int num, char * text);
+void newtListboxSetCurrentByKey(newtComponent co, void * key);
+void newtListboxSetText(newtComponent co, int num, const char * text);
+void newtListboxSetEntry(newtComponent co, int num, const char * text);
 void newtListboxSetWidth(newtComponent co, int width);
 /* return the data passed to AddEntry */
 void newtListboxSetData(newtComponent co, int num, void * data);
-int newtListboxAddEntry(newtComponent co, char * text, void * data);
-int newtListboxInsertEntry(newtComponent co, char * text, void * data, int num);
-int newtListboxDeleteEntry(newtComponent co, int num);
+int newtListboxAddEntry(newtComponent co, const char * text, const void * data);
+/* Send the key to insert after, or NULL to insert at the top */
+int newtListboxInsertEntry(newtComponent co, const char * text, const void * data, void * key);
+int newtListboxDeleteEntry(newtComponent co, void * data);
 void newtListboxClear(newtComponent co); /* removes all entries from listbox */
 void newtListboxGetEntry(newtComponent co, int num, char **text, void **data);
 /* Returns an array of data pointers from items, last element is NULL */
@@ -150,6 +157,8 @@ newtComponent newtTextbox(int left, int top, int with, int height, int flags);
 void newtTextboxSetText(newtComponent co, const char * text);
 void newtTextboxSetHeight(newtComponent co, int height);
 int newtTextboxGetNumLines(newtComponent co);
+char * newtReflowText(char * text, int width, int flexDown, int flexUp,
+		      int * actualWidth, int * actualHeight);
 
 struct newtExitStruct {
     enum { NEWT_EXIT_HOTKEY, NEWT_EXIT_COMPONENT } reason;
@@ -159,7 +168,7 @@ struct newtExitStruct {
     } u;
 } ;
 
-newtComponent newtForm(newtComponent vertBar, char * help, int flags);
+newtComponent newtForm(newtComponent vertBar, const char * help, int flags);
 newtComponent newtFormGetCurrent(newtComponent co);
 void newtFormSetBackground(newtComponent co, int color);
 void newtFormSetCurrent(newtComponent co, newtComponent subco);
@@ -172,15 +181,16 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es);
 void newtDrawForm(newtComponent form);
 void newtFormAddHotKey(newtComponent co, int key);
 
-newtComponent newtEntry(int left, int top, char * initialValue, int width,
+newtComponent newtEntry(int left, int top, const char * initialValue, int width,
 			char ** resultPtr, int flags);
-void newtEntrySet(newtComponent co, char * value, int cursorAtEnd);
+void newtEntrySet(newtComponent co, const char * value, int cursorAtEnd);
 void newtEntrySetFlags(newtComponent co, int flags, enum newtFlagsSense sense);
 
 newtComponent newtScale(int left, int top, int width, long long fullValue);
 void newtScaleSet(newtComponent co, unsigned long long amount);
 
 void newtComponentAddCallback(newtComponent co, newtCallback f, void * data);
+void newtComponentTakesFocus(newtComponent co, int val);
 
 /* this also destroys all of the components (including other forms) on the 
    form */
@@ -219,6 +229,50 @@ void newtFormDestroy(newtComponent form);
 #define NEWT_KEY_F11			NEWT_KEY_EXTRA_BASE + 111
 #define NEWT_KEY_F12			NEWT_KEY_EXTRA_BASE + 112
 
+/* not really a key, but newtGetKey returns it */
+#define NEWT_KEY_RESIZE			NEWT_KEY_EXTRA_BASE + 113
+
+#define NEWT_ANCHOR_LEFT		(1 << 0)
+#define NEWT_ANCHOR_RIGHT		(1 << 1)
+#define NEWT_ANCHOR_TOP			(1 << 2)
+#define NEWT_ANCHOR_BOTTOM		(1 << 3)
+
+#define NEWT_GRID_FLAG_GROWX		(1 << 0)
+#define NEWT_GRID_FLAG_GROWY		(1 << 1)
+
+typedef struct grid_s * newtGrid;
+enum newtGridElement { NEWT_GRID_EMPTY = 0,
+		       NEWT_GRID_COMPONENT, NEWT_GRID_SUBGRID };
+
+newtGrid newtCreateGrid(int cols, int rows);
+void newtGridSetField(newtGrid grid, int col, int row, 
+		      enum newtGridElement type, void * val, int padLeft,
+		      int padTop, int padRight, int padBottom, int anchor,
+		      int flags);
+void newtGridPlace(newtGrid grid, int left, int top);
+void newtGridFree(newtGrid grid, int recurse);
+void newtGridGetSize(newtGrid grid, int * width, int * height);
+void newtGridWrappedWindow(newtGrid grid, char * title);
+
+/* convienve */
+newtGrid newtButtonBar(char * button1, newtComponent * b1comp, ...);
+
+/* automatically centered and shrink wrapped */
+void newtWinMessage(char * title, char * buttonText, char * text, ...);
+void newtWinMessagev(char * title, char * buttonText, char * text, 
+		     va_list argv);
+
+/* Returns 0 if button 1 is pressed, 1 if button2 is pressed, 2 if F12
+   is pressed. If button2 is NULL, this behaves like newtWinMessage */
+int newtWinChoice(char * title, char * button1, char * button2, 
+		   char * text, ...);
+
+/* Returns 0 if F12 was pressed, 1 for button1, 2 for button2, 
+   3 for button3 */
+int newtWinTernary(char * title, char * button1, char * button2, 
+		   char * button3, char * message, ...);
+
+
 #ifdef __cplusplus
 } /* End of extern "C" { */
 #endif
