@@ -31,6 +31,7 @@ struct newtComponent_struct {
     int height, width; 
     int top, left;
     int takesFocus;
+    int isMapped;
 
     struct componentOps * ops;
 
@@ -64,6 +65,7 @@ struct componentOps {
     void (* draw)(newtComponent c);
     struct eventResult (* event)(newtComponent c, struct event ev);
     void (* destroy)(newtComponent c);
+    void (* place)(newtComponent c);
 } ;
 
 struct eventResult newtDefaultEventHandler(newtComponent c,
